#!/opt/cyops-search/.env/bin/python
""" Copyright start
  Copyright (C) 2008 - 2025 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """
import argparse
from util.connection import ElasticsearchConnection
from helpers.es_snapshot_helper import \
    (create_repository, delete_repository, take_snapshot,
     delete_snapshot, restore_snapshot, delete_indices)

es = None


def create_backup_repo():
    res = create_repository('es_bkp', {"type": "fs", "settings": {
                            "location": "/var/lib/elasticsearch/es_bkp"}}, es)
    if res:
        print("Snapshot repository successfully created")
        return True
    else:
        print("Failed to create snapshot repository")
        return False


def delete_backup_repository():
    res = delete_repository('es_bkp', es)
    if res:
        print("Snapshot repository successfully deleted")
        return True
    else:
        print("Failed to delete snapshot repository")
        return False


def create_snapshot():
    res = take_snapshot('es_bkp', 'es_cyops', {
                        "indices": "cyops_*", "expand_wildcards": "all"}, es)
    if res:
        print("snapshot successful")
        return True
    else:
        print("snapshot failed")
        return False


def remove_snapshot():
    res = delete_snapshot('es_bkp', 'es_cyops', es)
    if res:
        print("delete snapshot successful")
        return True
    else:
        print("delete snapshot failed")
        return False


def restore_es_data():
    res = restore_snapshot('es_bkp', 'es_cyops',
                           {"indices": "cyops_*", "index_settings": {"index.number_of_replicas": 1}}, es)
    if res:
        print("snapshot restore successful")
        return True
    else:
        print("snapshot restore failed")
        return False


def delete_all_indices():
    res = delete_indices('_all', es)
    if res:
        print("Deleted all indices")
        return True
    else:
        print("indices deletion failed")
        return False


def main():
    global es
    con = ElasticsearchConnection()
    es = con.create()
    parser = argparse.ArgumentParser()
    group = parser.add_mutually_exclusive_group()
    group.add_argument('--setup-backup-repository', dest='create_backup_repo', action='store_true',
                       help='Adds the default backup repository for the local ElasticSearch instance')
    group.add_argument('--delete-backup-repository', dest='delete_backup_repository', action='store_true',
                       help='Delete the default backup repository for the local ElasticSearch instance')
    group.add_argument('--create-snapshot', dest='create_snapshot', action='store_true',
                       help='Create the snapshot for all indices of local ElasticSearch instance')
    group.add_argument('--remove-snapshot', dest='remove_snapshot', action='store_true',
                       help='remove the snapshot for all indices of local ElasticSearch instance')
    group.add_argument('--restore-es-data', dest='restore_es_data', action='store_true',
                       help='restore the snapshot for all indices of local ElasticSearch instance')
    group.add_argument('--delete-all-indices', dest='delete_all_indices', action='store_true',
                       help='delete all indices of local ElasticSearch instance')
    args = parser.parse_args()
    if args.create_backup_repo:
        create_backup_repo()
    elif args.delete_backup_repository:
        delete_backup_repository()
    elif args.create_snapshot:
        create_snapshot()
    elif args.remove_snapshot:
        remove_snapshot()
    elif args.restore_es_data:
        restore_es_data()
    elif args.delete_all_indices:
        delete_all_indices()
    return


if __name__ == '__main__':
    main()
