#!/bin/bash
# Copyright start
# Copyright (C) 2008 - 2025 Fortinet Inc.
# All rights reserved.
# FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
# Copyright end

tune_logger_setting_for_elasticsearch() {
    local log4j_file=/etc/elasticsearch/log4j2.properties
    local max_size_per_file=10MB
    local max_file_count=10

    # Below is for elasticsearch.log
    # Default behavior:
    # 1. If IfAccumulatedFileSize for all elasticsearch.logs exceeds 2GB, then rotate.
    # New behavior:
    # 1. If IfAccumulatedFileCount exceeds $max_file_count, then rotate. Max size for every elaticsearch.log
    # can be $max_size_per_file.

    sed -i "s/appender.rolling_old.policies.size.size = 128MB/appender.rolling_old.policies.size.size = $max_size_per_file/g" $log4j_file
    sed -i "s/appender.rolling_old.strategy.action.condition.nested_condition.type = IfAccumulatedFileSize/appender.rolling_old.strategy.action.condition.nested_condition.type = IfAccumulatedFileCount/g" $log4j_file
    sed -i "s/appender.rolling_old.strategy.action.condition.nested_condition.exceeds = 2GB/appender.rolling_old.strategy.action.condition.nested_condition.exceeds = $max_file_count/g" $log4j_file

    # Below is for elasticsearch.json
    # Default behavior:
    # 1. If IfAccumulatedFileSize for all elasticsearch_server.json exceeds 2GB, then rotate.
    # New behavior:
    # 1. If IfAccumulatedFileCount exceeds $max_file_count, then rotate. Max size for every elasticsearch_server.json
    # can be $max_size_per_file.

    sed -i "s/appender.rolling.policies.size.size = 128MB/appender.rolling.policies.size.size = $max_size_per_file/g" $log4j_file
    sed -i "s/appender.rolling.strategy.action.condition.nested_condition.type = IfAccumulatedFileSize/appender.rolling.strategy.action.condition.nested_condition.type = IfAccumulatedFileCount/g" $log4j_file
    sed -i "s/appender.rolling.strategy.action.condition.nested_condition.exceeds = 2GB/appender.rolling.strategy.action.condition.nested_condition.exceeds = $max_file_count/g" $log4j_file

    # Below is for elasticsearch_deprecation.json
    # Default behavior:
    # 1. Rotate elasticsearch_deprecation.json if file size exceeds 1GB.
    # 2. Total files can be 4.
    # New behavior:
    # 1. Rotate elasticsearch_deprecation.json if file size exceeds $max_size_per_file.
    # 2. Total files can be 4. (no change)

    sed -i "s/appender.deprecation_rolling.policies.size.size = 1GB/appender.deprecation_rolling.policies.size.size = $max_size_per_file/g" $log4j_file

    # Below is for elasticsearch_deprecation.log
    # Default behavior:
    # 1. Rotate elasticsearch_deprecation.log if file size exceeds 1GB.
    # 2. Total files can be 4.
    # New behavior:
    # 1. Rotate elasticsearch_deprecation.log if file size exceeds $max_size_per_file.
    # 2. Total files can be 4. (no change)

    sed -i "s/appender.deprecation_rolling_old.policies.size.size = 1GB/appender.deprecation_rolling_old.policies.size.size = $max_size_per_file/g" $log4j_file

    # Below is for elasticsearch_index_search_slowlog.json
    # Default behavior:
    # 1. Rotate elasticsearch_index_search_slowlog.json if file size exceeds 1GB.
    # 2. Total files can be 4.
    # New behavior:
    # 1. Rotate elasticsearch_index_search_slowlog.json if file size exceeds $max_size_per_file.
    # 2. Total files can be 4. (no change)

    sed -i "s/appender.index_search_slowlog_rolling.policies.size.size = 1GB/appender.index_search_slowlog_rolling.policies.size.size = $max_size_per_file/g" $log4j_file

    # Below is for elasticsearch__index_search_slowlog.log
    # Default behavior:
    # 1. Rotate elasticsearch_index_search_slowlog.log if file size exceeds 1GB.
    # 2. Total files can be 4.
    # New behavior:
    # 1. Rotate elasticsearch_index_search_slowlog.log if file size exceeds $max_size_per_file.
    # 2. Total files can be 4. (no change)

    sed -i "s/appender.index_search_slowlog_rolling_old.policies.size.size = 1GB/appender.index_search_slowlog_rolling_old.policies.size.size = $max_size_per_file/g" $log4j_file

    # Below is for elasticsearch_index_indexing_slowlog.json
    # Default behavior:
    # 1. Rotate elasticsearch_index_indexing_slowlog.json if file size exceeds 1GB.
    # 2. Total files can be 4.
    # New behavior:
    # 1. Rotate elasticsearch_index_indexing_slowlog.json if file size exceeds $max_size_per_file.
    # 2. Total files can be 4. (no change)

    sed -i "s/appender.index_indexing_slowlog_rolling.policies.size.size = 1GB/appender.index_indexing_slowlog_rolling.policies.size.size = $max_size_per_file/g" $log4j_file

    # Below is for elasticsearch_index_indexing_slowlog.log
    # Default behavior:
    # 1. Rotate elasticsearch_index_indexing_slowlog.log if file size exceeds 1GB.
    # 2. Total files can be 4.
    # New behavior:
    # 1. Rotate elasticsearch_index_indexing_slowlog.log if file size exceeds $max_size_per_file.
    # 2. Total files can be 4. (no change)

    sed -i "s/appender.index_indexing_slowlog_rolling_old.policies.size.size = 1GB/appender.index_indexing_slowlog_rolling_old.policies.size.size = $max_size_per_file/g" $log4j_file

    # Avoid to log every info and waring and log to only waning.
    log_level=$(grep "rootLogger.level" $log4j_file | awk {'print $3'})
    if [ $log_level == "info" ]; then
        sed -E -i "s/(rootLogger.level\s=\s)info/\1warn/" $log4j_file
    fi
}

# Not require the ingest.geoip.downloader.enabled feature and save some resources consumption
disable_geoip(){
    local s_geoip="ingest.geoip.downloader.enabled"
    local s_state=$(grep -w "$s_geoip" $f_elasticsearch_yml | awk {'print $2'})
    if [ ! -z "$s_state" ] && [ "$s_state" = "false" ]; then
        #
        # In restore the db sub-command runs the setup_elastic.sh file with input "upgrade", etc, parameters in \
        # restore_password_es function. So disable_geoip function runs and \
        # avoid to make duplicate entry of "ingest.geoip.downloader.enabled" \
        # in /etc/elasticsearch/elasticsearch.yml file.
        #
        return
    fi
    if [ ! -z "$s_state" ] && [ "$s_state" != "false" ]; then
        # c in sed command: Replace complete line
        sed -i "/$s_geoip/c$s_geoip:\ false" $f_elasticsearch_yml
        return
    fi
    echo "$s_geoip: false" >> $f_elasticsearch_yml
}

set_java_home_globally() {
   local system_java_home_path="$java_home"
   local global_environment_file=/etc/environment
   if grep -qE "^\s*ES_JAVA_HOME=" $global_environment_file; then
        echo "ES_JAVA_HOME already set"
        return
   fi
    echo "ES_JAVA_HOME=$system_java_home_path" >> $global_environment_file
}

set_java_home_for_elasticsearch() {
    local system_java_home_path="$java_home"
    local es_conf_file=/etc/sysconfig/elasticsearch
    if grep -qE "^\s*ES_JAVA_HOME=" $es_conf_file; then
        echo "ES_JAVA_HOME already set"
    else
        #Add ES_JAVA_HOME
        echo "ES_JAVA_HOME=$system_java_home_path" >> $es_conf_file
    fi
    set_java_home_globally
}

remove_unused_modules() {
    modules_to_remove="repository-s3 \
                       repository-azure \
                       x-pack-ml \
                       x-pack-esql "
    for module in $modules_to_remove; do
        rm -rf /usr/share/elasticsearch/modules/"$module"
    done
}

create_snapshot_dir() {
    mkdir -p "$elastic_backup_path"
    chown -R elasticsearch:elasticsearch "$elastic_backup_path"
}

create_snapshot_repository() {
    if grep -Fxq "path.repo: /var/lib/elasticsearch/backups" $f_elasticsearch_yml; then
      echo "Old repo path detected correcting backup repository path"
      sed -i 's#path.repo:\ \/var\/lib\/elasticsearch\/backups#path.repo:\ \[\"\/var\/lib\/elasticsearch\/es_bkp\"\]#g' $f_elasticsearch_yml
    fi
    if grep -Fxq "path.repo: [\"/var/lib/elasticsearch/es_bkp\"]" $f_elasticsearch_yml; then
        echo "backup repository is already set for elasticsearch"
    else
        echo "enabling security for elasticsearch"
        echo "path.repo: [\"/var/lib/elasticsearch/es_bkp\"]" >> $f_elasticsearch_yml
    fi
}

update_es_certs() {
    # Elasticsearch 8.x onwards has a dedicated certs folder
    if [ ! -f /etc/elasticsearch/certs/server.leaf.crt ]; then
      echo "Updating certs"
      # In case of fresh install
      yes | cp /etc/pki/cyops/server.leaf.crt /etc/elasticsearch/certs/server.leaf.crt
      yes | cp /etc/pki/cyops/server.leaf.key /etc/elasticsearch/certs/server.leaf.key
      chmod g=u /etc/elasticsearch/certs/server.leaf.*
    fi
}

### Enabling ES Security
configure_es_8x_security() {
    # Elastic 8.x onwards, security is enabled by default. So we need to change the required keys only
    # Remove Defaults (keystore and truststore)
    sed -i '/keystore.path:\|truststore.path\|key\|certificate:/d' $f_elasticsearch_yml
    echo "enabling ssl and transport security for elasticsearch"
    if grep -q "supported_protocols: TLSv1.3" "$f_elasticsearch_yml"; then
        echo "TLSv1.3 only mode is already enabled"
    else
      echo "Enabling TLSv1.3 only mode"
      sed -i '/xpack.security.http.ssl:$/a \ \ \supported_protocols: TLSv1.3' $f_elasticsearch_yml
    fi
    echo "Updating certificate path"
    sed -i '/xpack.security.http\|transport.ssl:$/a \ \ certificate: /etc/elasticsearch/certs/server.leaf.crt' $f_elasticsearch_yml
    echo "Updating key path"
    sed -i '/xpack.security.http\|transport.ssl:$/a \ \ key: /etc/elasticsearch/certs/server.leaf.key' $f_elasticsearch_yml
    sed -i 's/^\s*cluster.initial_master_nodes:/# &/' $f_elasticsearch_yml
    # Set verification_mode to none
    sed -i '/verification_mode/d' $f_elasticsearch_yml
    sed -i '/xpack.security.http\|transport.ssl:$/a \ \ verification_mode: none' $f_elasticsearch_yml
}

enable_transport_ssl() {
    # Elastic 8.x onwards Auto configuration already enables transport security
    # Elastic 8.x security configuration is between strings "BEGIN SECURITY AUTO CONFIGURATION" and "END SECURITY AUTO CONFIGURATION"
    if ! grep -q  "BEGIN SECURITY AUTO CONFIGURATION" "$f_elasticsearch_yml"; then
        # Enabling transport security for upgrades from 7.17 or previous versions of elastic which has old elasticsearch.yml
        if grep -Fxq "xpack.security.transport.ssl.enabled:  true" "$f_elasticsearch_yml"; then
          echo "transport security is already enabled for elasticsearch"
        else
          echo "enabling transport security for elasticsearch"
          {
            echo "xpack.security.transport.ssl.enabled:  true"
          } >>"$f_elasticsearch_yml"
        fi

        # Update certs location in case of upgrade
        if ! grep -q "xpack.security.http.ssl.\(certificate|key\).*" "$f_elasticsearch_yml"; then
            sed -i '/xpack.security.http.ssl.certificate/d' "$f_elasticsearch_yml"
            sed -i '/xpack.security.http.ssl.key/d' "$f_elasticsearch_yml"
            {
              echo "xpack.security.http.ssl.certificate: /etc/elasticsearch/certs/server.leaf.crt"
              echo "xpack.security.http.ssl.key: /etc/elasticsearch/certs/server.leaf.key"
            } >>"$f_elasticsearch_yml"
        fi

        # Add transport.ssl cert if not present
        if ! grep -q "xpack.security.transport.ssl.\(certificate|key\).*" "$f_elasticsearch_yml"; then
            sed -i '/xpack.security.transport.ssl.certificate/d' "$f_elasticsearch_yml"
            sed -i '/xpack.security.transport.ssl.key/d' "$f_elasticsearch_yml"
            {
              # In case of upgrade, need to add key and cert since it does not exist previously
              echo "xpack.security.transport.ssl.certificate: /etc/elasticsearch/certs/server.leaf.crt"
              echo "xpack.security.transport.ssl.key: /etc/elasticsearch/certs/server.leaf.key"
            } >>"$f_elasticsearch_yml"
        fi
        if ! grep -q "xpack.security.transport.ssl.verification_mode.*" "$f_elasticsearch_yml"; then
            {
              # Set verification_mode to none
              echo "xpack.security.http.ssl.verification_mode: none"
              echo "xpack.security.transport.ssl.verification_mode: none"
            } >>"$f_elasticsearch_yml"
        fi
    fi
}

enable_external_connection() {
    if grep -Fxq "network.host: 0.0.0.0" "$f_elasticsearch_yml"; then
        echo "external connection configured"
    else
        echo "Enabling ElasticSearch For External Connections"
        sed -i 's/#network.host: 192.168.0.1/network.host: 0.0.0.0/g' "$f_elasticsearch_yml"
        echo "Enabling discovery type as single node"
        sed -i '/#http.port: 9200/a discovery.type: single-node' "$f_elasticsearch_yml"
    fi
}

disable_auto_create_index() {
    if grep -Fxq "action.auto_create_index: false" "$f_elasticsearch_yml"; then
        echo "Auto index creation already set to false"
    else
        echo "Disabling auto create index"
        sed -i -e '$aaction.auto_create_index: false' "$f_elasticsearch_yml"
        echo "Disabled auto create index"
    fi
}

enable_wildcard_index_deletion() {
    if grep -Fxq "action.destructive_requires_name: false" "$f_elasticsearch_yml"; then
        echo "Wildcard deletion already enabled"
    else
        echo "Enabling wildcard deletion"
        sed -i 's/#action.destructive_requires_name: true/action.destructive_requires_name: false/g' "$f_elasticsearch_yml"
        sed -i 's/#action.destructive_requires_name: false/action.destructive_requires_name: false/g' "$f_elasticsearch_yml"
        echo "Wildcard deletion enabled"
        systemctl restart elasticsearch
        sleep 15
    fi
}


#######################
## MAIN STARTS HERE ###
#######################

install_type=$1

# -------------------------- Constants -------------------------------------
f_elasticsearch_yml=/etc/elasticsearch/elasticsearch.yml
d_elasticsearch_base=/etc/elasticsearch
elastic_backup_path=/var/lib/elasticsearch/es_bkp
f_tmp_location=/usr/share/elasticsearch/tmp
es_conf_file=/etc/sysconfig/elasticsearch
global_environment_file=/etc/environment
java_home=/usr
# -------------------------- Constants End ---------------------------------

# -- Setup tmp location ----------------------------------------------------
mkdir -p $f_tmp_location
chown -R elasticsearch:elasticsearch $f_tmp_location
chmod -R 755 $f_tmp_location

if grep -Fxq "ES_TMPDIR=$f_tmp_location" "$es_conf_file"; then
    echo "ES_TMPDIR already set"
else
    echo "ES_TMPDIR=$f_tmp_location" >>$es_conf_file
    echo "ES_TMPDIR=$f_tmp_location" >>$global_environment_file
fi
# -- Setup tmp location End -------------------------------------------------


## Copy security.policy and fsr.options
yes | cp /opt/cyops/configs/cyops-search/security.policy /etc/elasticsearch/security.policy
yes | cp /opt/cyops/configs/cyops-search/fsr.options /etc/elasticsearch/jvm.options.d/fsr.options
chown elasticsearch:elasticsearch /etc/elasticsearch/security.policy
chown elasticsearch:elasticsearch /etc/elasticsearch/jvm.options.d/fsr.options

## Set JAVA Home
set_java_home_for_elasticsearch

## Remove unused modules
remove_unused_modules

### Fresh
if [ "$install_type" = "fresh" ]; then
    ## Setup Elasticsearch

    create_snapshot_dir

    create_snapshot_repository

    tune_logger_setting_for_elasticsearch

    disable_geoip

    enable_external_connection

    ## Enabling security
    configure_es_8x_security

    update_es_certs

    disable_auto_create_index
    enable_wildcard_index_deletion

    . $es_conf_file
    . $global_environment_file

    # Changing ownership
    chown root:elasticsearch -R /etc/elasticsearch/
    chown elasticsearch:elasticsearch -R /var/lib/elasticsearch
fi

### Upgrade
if [ "$install_type" = "upgrade" ]; then
    # Move the certs to dedicated certs folder
    if [ -f /etc/elasticsearch/server.leaf.crt ]; then
        echo "Moving  elasticsearch certs"
        # Create certs dir in case of upgrade
        mkdir -p /etc/elasticsearch/certs
        yes | mv /etc/elasticsearch/server.leaf.crt /etc/elasticsearch/certs/server.leaf.crt
        yes | mv /etc/elasticsearch/server.leaf.key /etc/elasticsearch/certs/server.leaf.key
        chmod g=u /etc/elasticsearch/certs/server.leaf.*
    fi
    create_snapshot_repository
    disable_auto_create_index
    enable_wildcard_index_deletion
    ## Required for older elasticsearch.yml
    enable_transport_ssl
fi
