# Copyright start
# Copyright (C) 2008 - 2025 Fortinet Inc.
# All rights reserved.
# FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
# Copyright end

set_default_user() {
  # set default user passwords
  export ES_JAVA_HOME=/usr
  # This is to check if env has ES_JAVA_HOME.
  # we have seen on cloud that this is not set/read so adding this for debug purpose
  cat /etc/environment | grep ES_JAVA_HOME
  if $(/usr/share/elasticsearch/bin/elasticsearch-keystore list | grep -q 'bootstrap.password'); then
    echo "bootstrap password already set"
  else
    echo "setting default users..."
    printf $i_password | /usr/share/elasticsearch/bin/elasticsearch-keystore add "bootstrap.password" -f
    exit_status=$?
    # Adding the below block in case when the elasticsearch does not find ES_JAVA_HOME.
    if [ $exit_status -ne 0 ];then
        echo "Setting up bootstrap password failed for elasticsearch"
        echo "Run the below command to set the same"
        echo "printf $(csadm license --get-device-uuid) | /usr/share/elasticsearch/bin/elasticsearch-keystore add "bootstrap.password" -f"
    fi
  fi
}

update_es_certs() {
  # Elasticsearch 8.x onwards has a dedicated certs folder
  if [ ! -f /etc/elasticsearch/certs/server.leaf.crt ]; then
    echo "Updating certs"
    # In case of fresh install
    yes | cp /etc/pki/cyops/server.leaf.crt /etc/elasticsearch/certs/server.leaf.crt
    yes | cp /etc/pki/cyops/server.leaf.key /etc/elasticsearch/certs/server.leaf.key
    chmod g=u /etc/elasticsearch/certs/server.leaf.*
  fi
}

set_elastic_backup_repository() {
    is_elastic_enabled=$(systemctl list-unit-files | grep enabled |grep elasticsearch |wc -l)
    if [ $is_elastic_enabled -eq 1 ]; then
      echo "Elasticsearch is enabled"
      systemctl restart elasticsearch
      sleep 15
      search_prefix=/opt/cyops-search/
      search_env=.env/bin/python
      es_utility=falcon/es_config_util.py
      ${search_prefix}${search_env} ${search_prefix}${es_utility} --setup-backup-repository
    fi
}

modify_elastic_config() {
  config_file=$1
  es_user_value=$(python3 $f_conf_util -f $config_file -g es_user)
  if [ "$es_user_value" = "None" ]; then
    python3 $f_conf_util -f "$config_file" -k es_user -v elastic
    python3 $f_conf_util -f "$config_file" -k secret -v "$i_password_encrypted"
    python3 $f_conf_util -f "$config_file" -k use_ssl -v true
  fi
}

#######################
## MAIN STARTS HERE ###
#######################

i_password=$1
i_password_encrypted=$2
install_type=$3
set -e

# -------------------------- Constants -------------------------------------
f_config=/opt/cyops/configs/database/db_config.yml
f_default_config=/opt/cyops/configs/database/db_config.yml.default
f_elasticsearch_yml="/etc/elasticsearch/elasticsearch.yml"
f_conf_util=/opt/cyops/scripts/confUtil.py
f_utility=/opt/cyops-search/falcon/utility.py
d_elasticsearch_base="/etc/elasticsearch"
b_python=/opt/cyops-search/.env/bin/python
# -------------------------- Constants End ---------------------------------



if [ "$install_type" = "fresh" ]; then
    echo "This is fresh install. Updating db_config.yml file"
    set_default_user
    modify_elastic_config $f_config
    update_es_certs
    set_elastic_backup_repository
fi

if [ "$install_type" = "upgrade" ]; then
    node_id=$(csadm license --get-device-uuid)
    csadm ha list-nodes --primary | grep -q "$node_id" &>/dev/null
    if [ $? == 0 ]; then
        echo "This is primary node. Updating db_config.yml file"
        modify_elastic_config $f_config

        echo "Propagating es password change to secondary nodes, if any"
        /bin/hamanager add-command --async --commandId 'update_es_credentials' --command "/opt/cyops-auth/utilities/ha/cmd_executor.py --command update-es-credentials --input-arg $i_password_encrypted"
    else
        echo "This is secondary node. updating db_config.yml.default file"
        modify_elastic_config $f_default_config
    fi
fi

python3 $f_conf_util -f $f_config -k use_ssl -v true